#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from setuptools import setup
from setuptools import find_packages

VERSION = "0.0.2"

with open("README.md", "r") as fh:
	long_description = fh.read()

setup(
		name         = "agine",
		version      = VERSION,
		author       = "Debmalya Pramanik",
		author_email = "",

		description                   = "Functionalities for Points in n-Dimensional Space",
		long_description              = long_description,
		long_description_content_type = "text/markdown",

		url         = "https://github.com/ZenithClown/agine",
		packages    = find_packages(),
		classifiers = [
			"Programming Language :: Python :: 3",
			"License :: OSI Approved :: Apache Software License"
		],
		python_requires  = ">=3.6",
		install_requires = ["numpy"]
	)